// resampler.js
// HB, 10-08-11 for Brain Innovation

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ DECLARATIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  
var scriptObj 					= new Object; 
scriptObj.scriptNameForUser 	= "resampler"; 
scriptObj.dialogFileName 		= "resampler.ui"; 
scriptObj.dialogAccessName      = "imcalcDlg"; 

// global variables
var command = "Unknown";
var incompleteinfo = false;
var hasposfile = false;
var hastrffile = false;
var openInBVQX = false;
var doBatch = false;

// file type IDs correspond to plugin converter's IDs
const FILE_TYPE_UNKNOWN = 0;
const NIFTI_FILE_TYPE = 1; 
const NIFTI_PAIR_FILE_TYPE = 2;
const ANALYZE75_FILE_TYPE = 3;
const NIFTI_IMG_TYPE = 4;
const BV_TYPE_VMR = 20;
const BV_TYPE_FMR = 21;
const BV_TYPE_AMR = 22;
const BV_TYPE_VTC = 24;
const BV_TYPE_VOI = 27;
const BV_TYPE_DMR = 29;
const BV_TYPE_VMP = 201;
const BV_TYPE_VMP_AR = 202;
const BV_TYPE_VMP_NR = 203;
const FILE_TYPE_TXT = 401;
const BV_TYPE_MAP = 200;
const BV_TYPE_STC = 23;
const BV_TYPE_DWI = 30;
const BV_TYPE_VDW = 31;
const BV_TYPE_GLM = 205;
const BV_TYPE_AVA = 206;
const BV_TYPE_PRT = 208;
const BV_TYPE_TRF = 301;
const BV_TYPE_POS = 302;
// submenu options
const PRINT_TO_LOG = 1;
const PRINT_TO_FILE = 2;
const PRINT_TO_LOG_AND_FILE = 3;
const PRINT_HIST = 4;
// 	enum ConvMenuOptions
const CONVERT_SINGLE_FILE = 1;
const CONVERT_BATCH = 2;
const CONVERT_REGISTERED = 3;
const CONVERT_REGISTERED_BATCH = 4;
// enum POSMENUOPTIONS
const POS_MATRIX_FROM_FILE = 1;
const POS_ADD_TO_FILE = 2;
const POS_BVQXPOS_FROM_NIIPOS = 3; // not active yet
const POS_COMPUTE_COORDS_OF_POINTS = 4; // not active yet
// enum TRFMENUOPTIONS
const TRF_INVERT_MAT        = 1;
const TRF_APPLY_TRF_MAN     = 2;
const TRF_APPLY_TRF_FILE    = 3;
const TRF_COMPUTE_BBOX      = 4;
const TRF_RESAMPLE          = 5;

// ~~~ nicknames for children ~~~
// global tab
var optConvertButton, optInfoButton, optPosButton, optTrfButton, optBatchButton, startButton;
// transformation tab
var invertTrfButton,applyManTrfButton, getTrfFileTypeBox, transformGrBox, 
t00, t01, t02, t03, t10, t11, t12, t13, t20, t21, t22, t23, t30, t31, t32, t33;
var newXdimFld, newYdimFld, newZdimFld, newX, newY, newZ, resampleButton; 
 // batch tab
var writeBatchFileButton;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ GENERIC FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  


scriptObj.errorfree = function() {

    var errornr = BrainVoyagerQX.GetPluginIntParameter("ErrorNr");
    if (errornr == 0) return true;
    else return false;
}

scriptObj.checkerrorstatus = function() {

    var errornr = BrainVoyagerQX.GetPluginIntParameter("ErrorNr");
    if (errornr < 0) {
            var errormsg = BrainVoyagerQX.GetPluginStringParameter("ErrorMsg");
            BrainVoyagerQX.PrintToLog(errormsg);
            cancel = true;
    } 
    return errornr;
}

scriptObj.start = function() {
    
    var success = true;   
    
    this.prepareTrfFuncs();
    
    // then start plugin
    if ((!incompleteinfo) && (success)) {
        BrainVoyagerQX.SetPluginStringParameter("Command", "Start");    
        BrainVoyagerQX.ExecutePlugin();
    
        // ask parameters from plugin
        if (this.errorfree()) {
            var origfmr = BrainVoyagerQX.ActiveDocument;
            origfmr.Close();
            var resampledname = BrainVoyagerQX.GetPluginStringParameter("ResampledFile");
            BrainVoyagerQX.PrintToLog("Opening " + resampledname + "...");
            var bvqxdoc = BrainVoyagerQX.OpenDocument(resampledname.toString());
        }
    } else {
        BrainVoyagerQX.PrintToLog("Cannot start function: incomplete information. Please check whether all parameters have been collected.");
    }   
    
    // now close the dialog
    this.imcalcDlg.close(); 
}



scriptObj.printMatrixToLog = function(matrix) {

    BrainVoyagerQX.PrintToLog("Matrix:");
    for (i=0; i<4; i++) {
            BrainVoyagerQX.PrintToLog(matrix[i][0] + ", " + matrix[i][1] + ", " + matrix[i][2] + ", " + matrix[i][3]); 
    }
}

scriptObj.onlyHasNumericValues = function(matrix) {

    var value;
    var hasOnlyNumericValues = true;
    for (i=0; i<4; i++) {
        for (j=0; j<4; j++) {
           if (isNaN(matrix[i][j])) {
                hasOnlyNumericValues = false;
                break;
            }
        }            
    }
    return hasOnlyNumericValues;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ TRANSFORMATION FUNCTIONS ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  


scriptObj.prepareTrfFuncs = function() {

    BrainVoyagerQX.PrintToLog("Prepare for plugin: retrieve dialog parameters...");
    
    // get information from widgets
    var submenuoption, matrix, fileid;
    if (invertTrfButton.checked == 1) {
        submenuoption = TRF_INVERT_MAT;
        matrix = this.getTrfMatrix();
        this.printMatrixToLog(matrix);
        if (!this.onlyHasNumericValues(matrix)) {
            BrainVoyagerQX.PrintToLog("Cannot proceed: values of transformation matrix are not numeric.");
            incompleteinfo = true;
        }        
    } else if (applyManTrfButton.checked == 1) {
        submenuoption = TRF_APPLY_TRF_MAN;
        matrix = this.getTrfMatrix();
        this.printMatrixToLog(matrix);
        if (!this.onlyHasNumericValues(matrix)) {
            BrainVoyagerQX.PrintToLog("Cannot proceed: values of positioning matrix are not numeric.");
            incompleteinfo = true;
        }        
        fileid = this.getFiletypeToTransform();
    } else if (resampleButton.checked == 1) {
    
        submenuoption = TRF_RESAMPLE;        
        newX = newXdimFld.text;
        newY = newYdimFld.text;
        newZ = newZdimFld.text;   
        if (newX.length==0 || newY.length==0 || newZ.length==0) {
            BrainVoyagerQX.PrintToLog("Cannot proceed: new dimension parameters required.");
            incompleteinfo = true;
        }        
        fileid = this.getFiletypeToTransform();
    } 

    // set parameters for plugin
    if (!incompleteinfo) {
    
        BrainVoyagerQX.PrintToLog("Set plugin parameters for transformation function...");
        BrainVoyagerQX.SetPluginStringParameter("MenuOption", "TrfFuncs"); 
        BrainVoyagerQX.SetPluginIntParameter("SubmenuOption", submenuoption);
        // use manual matrix
        if (submenuoption == 1 || submenuoption == 2) {
            BrainVoyagerQX.PrintToLog("Set matrix fields...");
            this.setMatrixParametersForPlugin(matrix);       
        }
        if (submenuoption == 2 || submenuoption == 3 || submenuoption == TRF_RESAMPLE) {
        
        BrainVoyagerQX.PrintToLog("Set file ID: " + fileid);
        
            BrainVoyagerQX.SetPluginIntParameter("FileID", fileid);
            
    /*        if (this.isniftifile(fromID)) {
                if (hasniftifile) {
                    BrainVoyagerQX.SetPluginStringParameter("NiftiFileName", niftiNameLabel.text); 
                    
           BrainVoyagerQX.PrintToLog("Set NIfTI name...");          
                    
                } else {
                    incompleteinfo = true;
                    
           BrainVoyagerQX.PrintToLog("incompleteinfo = true...");

                }
            }  
      */                
        }        
        if (submenuoption == TRF_RESAMPLE) {
            BrainVoyagerQX.SetPluginIntParameter("NewDimX", newX);
            BrainVoyagerQX.SetPluginIntParameter("NewDimY", newY);
            BrainVoyagerQX.SetPluginIntParameter("NewDimZ", newZ);
        }
    } else {
        BrainVoyagerQX.PrintToLog("Information not complete.");
    }
}

scriptObj.getTrfMatrix = function() {

	BrainVoyagerQX.PrintToLog("Get transformation matrix...");
    var matrix = [[0,0,0,0],[0,0,0,0],[0,0,0,0],[0,0,0,0]]; 
    matrix[0][0] = t00.text;
    matrix[0][1] = t01.text;
    matrix[0][2] = t02.text;
    matrix[0][3] = t03.text;
    matrix[1][0] = t10.text;
    matrix[1][1] = t11.text;
    matrix[1][2] = t12.text;
    matrix[1][3] = t13.text;
    matrix[2][0] = t20.text;
    matrix[2][1] = t21.text;
    matrix[2][2] = t22.text;
    matrix[2][3] = t23.text;
    matrix[3][0] = t30.text;
    matrix[3][1] = t31.text;
    matrix[3][2] = t32.text;
    matrix[3][3] = t33.text;    
    return matrix;
}

scriptObj.setMatrixParametersForPlugin = function(matrix) {

    BrainVoyagerQX.SetPluginFloatParameter("m00", matrix[0][0]);
    BrainVoyagerQX.SetPluginFloatParameter("m01", matrix[0][1]);
    BrainVoyagerQX.SetPluginFloatParameter("m02", matrix[0][2]);
    BrainVoyagerQX.SetPluginFloatParameter("m03", matrix[0][3]);

    BrainVoyagerQX.SetPluginFloatParameter("m10", matrix[1][0]);
    BrainVoyagerQX.SetPluginFloatParameter("m11", matrix[1][1]);
    BrainVoyagerQX.SetPluginFloatParameter("m12", matrix[1][2]);
    BrainVoyagerQX.SetPluginFloatParameter("m13", matrix[1][3]);

    BrainVoyagerQX.SetPluginFloatParameter("m20", matrix[2][0]);
    BrainVoyagerQX.SetPluginFloatParameter("m21", matrix[2][1]);
    BrainVoyagerQX.SetPluginFloatParameter("m22", matrix[2][2]);
    BrainVoyagerQX.SetPluginFloatParameter("m23", matrix[2][3]);

    BrainVoyagerQX.SetPluginFloatParameter("m30", matrix[3][0]);
    BrainVoyagerQX.SetPluginFloatParameter("m31", matrix[3][1]);
    BrainVoyagerQX.SetPluginFloatParameter("m32", matrix[3][2]);
    BrainVoyagerQX.SetPluginFloatParameter("m33", matrix[3][3]);
}

scriptObj.getFiletypeToTransform = function() {
  
    var filetypeid;
    var currentindex = getTrfFileTypeBox.currentIndex;
    switch (currentindex) {
        case 0: filetypeid = FILE_TYPE_UNKNOWN; break;
        case 1: filetypeid = BV_TYPE_VMR; break;
        case 2: filetypeid = BV_TYPE_FMR; break;
        case 3: filetypeid = BV_TYPE_DMR; break;
        case 4: filetypeid = BV_TYPE_VTC; break;
        case 5: filetypeid = BV_TYPE_VMP; break;
  //      case 6: filetypeid = NIFTI_FILE_TYPE; break;
  //      case 7: filetypeid = NIFTI_PAIR_FILE_TYPE; break;
        default: filetypeid = FILE_TYPE_UNKNOWN;
    }
    return filetypeid;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ MAIN ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  

scriptObj.initDlg = function() { 

    this.imcalcDlg.windowTitle = "resampler"; 

    // ~~~ make nicknames for children ~~~
	var bvqx = BrainVoyagerQX;
    startButton = bvqx.FindChild("btnStart"); 

    // transformation tab
    invertTrfButton = bvqx.FindChild("rbtnInvertTrf");
    applyManTrfButton = bvqx.FindChild("rbtnApplyManTrf");
  //  applyBVQXTrfButton = bvqx.FindChild("rbtnApplyBVQXtrf");
    getTrfFileTypeBox = bvqx.FindChild("cmBoxTrfFiles");
    transformGrBox = bvqx.FindChild("grboxTrfMat");
//    getTrfFileButton = bvqx.FindChild("btnGetTrfFile");
    trfFileLabel = bvqx.FindChild("lblTrfFile");
    newXdimFld = bvqx.FindChild("leNewX");
    newYdimFld = bvqx.FindChild("leNewY");
    newZdimFld = bvqx.FindChild("leNewZ");
    resampleButton = bvqx.FindChild("rbtnResample");
    t00 = bvqx.FindChild("le00"); 
    t01 = bvqx.FindChild("le01"); 
    t02 = bvqx.FindChild("le02"); 
    t03 = bvqx.FindChild("le03"); 
    t10 = bvqx.FindChild("le10"); 
    t11 = bvqx.FindChild("le11"); 
    t12 = bvqx.FindChild("le12"); 
    t13 = bvqx.FindChild("le13"); 
    t20 = bvqx.FindChild("le20"); 
    t21 = bvqx.FindChild("le21"); 
    t22 = bvqx.FindChild("le22"); 
    t23 = bvqx.FindChild("le23"); 
    t30 = bvqx.FindChild("le30"); 
    t31 = bvqx.FindChild("le31"); 
    t32 = bvqx.FindChild("le32"); 
    t33 = bvqx.FindChild("le33");     
  
    // connect widgets to functions here  
  //  getTrfFileButton.clicked.connect(this, this.getTrfFile);
    startButton.clicked.connect(this, this.start);
   
}

returnScriptObj = function() { 
	return scriptObj; 
} 

returnScriptObj(); // should be after returnScriptOjb = function()
